@@ -20,7 +20,7 @@
  *              This worker is deprecated                                  *
  * Author:      Gal Shachor <shachor@il.ibm.com>                           *
  * Based on:    jserv_ajpv12.c from Jserv                                  *
- * Version:     $Revision: 603324 $                                          *
+ * Version:     $Revision: 747878 $                                          *
  ***************************************************************************/
 
 #include "jk_ajp12_worker.h"
@@ -127,7 +127,7 @@ static int JK_METHOD service(jk_endpoint_t *e,
          attempt++) {
         p->sd =
             jk_open_socket(&p->worker->worker_inet_addr,
-                           JK_FALSE, 0, 0, l);
+                           JK_FALSE, 0, 0, 0, l);
 
         jk_log(l, JK_LOG_DEBUG, "In jk_endpoint_t::service, sd = %d",
                p->sd);
@@ -191,8 +191,8 @@ static int JK_METHOD validate(jk_worker_t *pThis,
                "In jk_worker_t::validate for worker %s contact is %s:%d",
                p->name, host, port);
 
-        if (port > 1024 && host) {
-            if (jk_resolve(host, port, &p->worker_inet_addr, l)) {
+        if (host) {
+            if (jk_resolve(host, port, &p->worker_inet_addr, we->pool, l)) {
                 return JK_TRUE;
             }
             jk_log(l, JK_LOG_ERROR,
@@ -280,7 +280,6 @@ int JK_METHOD ajp12_worker_factory(jk_worker_t **w,
                 private_data->worker.get_endpoint = get_endpoint;
                 private_data->worker.destroy = destroy;
                 private_data->worker.maintain = NULL;
-                private_data->worker.retries = JK_RETRIES;
 
                 *w = &private_data->worker;
                 return JK_AJP12_WORKER_TYPE;
@@ -540,7 +539,7 @@ static int ajpv12_handle_response(ajp12_endpoint_t * p,
         char *line = NULL;
         char *name = NULL;
         char *value = NULL;
-#ifdef _REENTRANT
+#ifdef _MT_CODE_PTHREAD
         char *lasts;
 #endif
 
@@ -588,7 +587,7 @@ static int ajpv12_handle_response(ajp12_endpoint_t * p,
         jk_log(l, JK_LOG_DEBUG, "ajpv12_handle_response, read %s=%s", name,
                value);
         if (0 == strcmp("Status", name)) {
-#ifdef _REENTRANT
+#ifdef _MT_CODE_PTHREAD
             char *numeric = strtok_r(value, " \t", &lasts);
 #else
             char *numeric = strtok(value, " \t");
@@ -600,7 +599,7 @@ static int ajpv12_handle_response(ajp12_endpoint_t * p,
                        "ajpv12_handle_response, invalid status code");
                 return JK_FALSE;
             }
-#ifdef _REENTRANT
+#ifdef _MT_CODE_PTHREAD
             reason = jk_pool_strdup(s->pool, strtok_r(NULL, " \t", &lasts));
 #else
             reason = jk_pool_strdup(s->pool, strtok(NULL, " \t"));
