@@ -19,7 +19,7 @@
  * Description: common stuff for bi-directional protocol ajp13/ajp14.      *
  * Author:      Gal Shachor <shachor@il.ibm.com>                           *
  * Author:      Henri Gomez <hgomez@apache.org>                            *
- * Version:     $Revision: 550221 $                                          *
+ * Version:     $Revision: 893085 $                                          *
  ***************************************************************************/
 
 #ifndef JK_AJP_COMMON_H
@@ -27,6 +27,7 @@
 
 #include "jk_service.h"
 #include "jk_msg_buff.h"
+#include "jk_shm.h"
 #include "jk_mt.h"
 
 #ifdef __cplusplus
@@ -34,6 +35,22 @@ extern "C"
 {
 #endif                          /* __cplusplus */
 
+#define JK_AJP_STATE_IDLE               (0)
+#define JK_AJP_STATE_OK                 (1)
+#define JK_AJP_STATE_ERROR              (2)
+#define JK_AJP_STATE_PROBE              (3)
+#define JK_AJP_STATE_DEF                (JK_AJP_STATE_IDLE)
+#define JK_AJP_STATE_TEXT_IDLE          ("OK/IDLE")
+#define JK_AJP_STATE_TEXT_OK            ("OK")
+#define JK_AJP_STATE_TEXT_ERROR         ("ERR")
+#define JK_AJP_STATE_TEXT_PROBE         ("ERR/PRB")
+#define JK_AJP_STATE_TEXT_MAX           (JK_AJP_STATE_PROBE)
+#define JK_AJP_STATE_TEXT_DEF           (JK_AJP_STATE_TEXT_IDLE)
+
+/* We accept doing global maintenance if we are */
+/* JK_AJP_MAINTAIN_TOLERANCE seconds early. */
+#define JK_AJP_MAINTAIN_TOLERANCE (2)
+
 /*
  * Conditional request attributes
  *
@@ -195,7 +212,14 @@ extern "C"
 #define AJP_DEF_REPLY_TIMEOUT     (0)   /* NO REPLY TIMEOUT                        */
 #define AJP_DEF_PREPOST_TIMEOUT   (0)   /* NO PREPOST TIMEOUT => NO CPING/CPONG    */
 #define AJP_DEF_RECOVERY_OPTS     (0)   /* NO RECOVERY / NO    */
-#define AJP_DEF_SOCKET_TIMEOUT    (0)  /* No timeout */
+#define AJP_DEF_SOCKET_TIMEOUT    (0)   /* No timeout */
+#define AJP_DEF_PING_TIMEOUT      (10000)  /* Default CPING/CPONG timeout (10 seconds) */
+
+#define AJP_CPING_NONE            (0)   /* Do not send cping packets */
+#define AJP_CPING_CONNECT         (1)   /* Send cping on fresh connection */
+#define AJP_CPING_PREPOST         (2)   /* Send cping before sending request */
+#define AJP_CPING_INTERVAL        (4)   /* Send cping on regular intervals */
+
 
 #define RECOVER_ABORT_IF_TCGETREQUEST    0x0001 /* DON'T RECOVER IF TOMCAT FAILS AFTER RECEIVING REQUEST */
 #define RECOVER_ABORT_IF_TCSENDHEADER    0x0002 /* DON'T RECOVER IF TOMCAT FAILS AFTER SENDING HEADERS */
@@ -232,11 +256,27 @@ typedef struct ajp_worker ajp_worker_t;
 
 struct ajp_worker
 {
+    jk_worker_t worker;
+    /* Shared memory worker data */
+    jk_shm_ajp_worker_t *s;
+
+    char         name[JK_SHM_STR_SIZ+1];
+    /* Sequence counter starting at 0 and increasing
+     * every time we change the config
+     */
+    volatile unsigned int sequence;
+
+    jk_pool_t p;
+    jk_pool_atom_t buf[TINY_POOL_SIZE];
+
+    JK_CRIT_SEC cs;
+
     struct sockaddr_in worker_inet_addr;    /* Contains host and port */
     unsigned connect_retry_attempts;
-    const char *name;
-    const char *host;
+    char host[JK_SHM_STR_SIZ+1];
     int port;
+    int addr_sequence;  /* Whether the address is resolved */
+    int maintain_time;
     /*
      * Open connections cache...
      *
@@ -244,10 +284,10 @@ struct ajp_worker
      * 2. Cache size.
      * 3. An array of "open" endpoints.
      */
-    JK_CRIT_SEC cs;
     unsigned int ep_cache_sz;
     unsigned int ep_mincache_sz;
     unsigned int ep_maxcache_sz;
+    int cache_acquire_timeout;
     ajp_endpoint_t **ep_cache;
 
     int proto;              /* PROTOCOL USED AJP13/AJP14 */
@@ -257,8 +297,6 @@ struct ajp_worker
     /* Weak secret similar with ajp12, used in ajp13 */
     const char *secret;
 
-    jk_worker_t worker;
-
     /*
      * Post physical connect handler.
      * AJP14 will set here its login handler
@@ -269,6 +307,7 @@ struct ajp_worker
      * Handle Socket Timeouts
      */
     int socket_timeout;
+    int socket_connect_timeout;
     int keepalive;
     int socket_buf;
     /*
@@ -279,17 +318,30 @@ struct ajp_worker
     /*
      * Handle Connection/Reply Timeouts
      */
-    int connect_timeout;   /* connect cping/cpong delay in ms (0 means disabled)  */
-    int reply_timeout;     /* reply timeout delay in ms (0 means disabled) */
-    int prepost_timeout;   /* before sending a request cping/cpong timeout delay in ms (0 means disabled) */
+    int connect_timeout;      /* connect cping/cpong delay in ms (0 means disabled)  */
+    int reply_timeout;        /* reply timeout delay in ms (0 means disabled) */
+    int prepost_timeout;      /* before sending a request cping/cpong timeout delay in ms (0 means disabled) */
+    int conn_ping_interval;   /* interval for sending keepalive cping packets on
+                               * unused connection */
+    int ping_timeout;         /* generic cping/cpong timeout. Used for keepalive packets or
+                               * as default for boolean valued connect and prepost timeouts.
+                               */
+    unsigned int ping_mode;   /* Ping mode flags (which types of cpings should be used) */
+    /*
+     * Recovery options
+     */
+    unsigned int recovery_opts;
 
     /*
-     * Recovery option
+     * Public property to enable the number of retry attempts
+     * on this worker.
      */
-    unsigned int recovery_opts; /* Set the recovery option */
-    
+    int retries;
+
     unsigned int max_packet_size;  /*  Maximum AJP Packet size */
-    
+
+    int retry_interval;            /*  Number of milliseconds to sleep before doing a retry */
+
     /* 
      * HTTP status that will cause failover (0 means disabled)
      */
@@ -321,6 +373,9 @@ struct ajp_endpoint
        handled by this endpoint */
     time_t last_access;
     int last_errno;
+    /* Last operation performed via this endpoint */
+    int last_op;
+    int addr_sequence;  /* Whether the address is resolved */
 };
 
 /*
@@ -342,6 +397,10 @@ struct ajp_operation
  */
 
 
+const char *jk_ajp_get_state(ajp_worker_t *aw, jk_logger_t *l);
+
+int jk_ajp_get_state_code(const char *v);
+
 int ajp_validate(jk_worker_t *pThis,
                  jk_map_t *props,
                  jk_worker_env_t *we, jk_logger_t *l, int proto);
@@ -350,6 +409,9 @@ int ajp_init(jk_worker_t *pThis,
              jk_map_t *props,
              jk_worker_env_t *we, jk_logger_t *l, int proto);
 
+int JK_METHOD ajp_worker_factory(jk_worker_t **w,
+                                 const char *name, jk_logger_t *l);
+
 int ajp_destroy(jk_worker_t **pThis, jk_logger_t *l, int proto);
 
 int JK_METHOD ajp_done(jk_endpoint_t **e, jk_logger_t *l);
@@ -361,6 +423,10 @@ int ajp_connect_to_endpoint(ajp_endpoint_t * ae, jk_logger_t *l);
 
 void ajp_close_endpoint(ajp_endpoint_t * ae, jk_logger_t *l);
 
+void jk_ajp_pull(ajp_worker_t * aw, int locked, jk_logger_t *l);
+
+void jk_ajp_push(ajp_worker_t * aw, int locked, jk_logger_t *l);
+
 int ajp_connection_tcp_send_message(ajp_endpoint_t * ae,
                                     jk_msg_buf_t *msg, jk_logger_t *l);
 
@@ -369,7 +435,10 @@ int ajp_connection_tcp_get_message(ajp_endpoint_t * ae,
 
 int JK_METHOD ajp_maintain(jk_worker_t *pThis, time_t now, jk_logger_t *l);
 
+int jk_ajp_get_cping_mode(const char *m, int def);
+
 #ifdef __cplusplus
 }
 #endif                          /* __cplusplus */
 #endif                          /* JK_AJP_COMMON_H */
+
