@@ -23,7 +23,7 @@
  * @package jk_connect
  * @author      Gal Shachor <shachor@il.ibm.com>
  * @author      Mladen Turk <mturk@apache.org>
- * @version     $Revision: 745136 $
+ * @version     $Revision: 915125 $
  */
 
 
@@ -42,6 +42,10 @@ static apr_pool_t *jk_apr_pool = NULL;
 /* FIONREAD on Solaris et al. */
 #include <sys/filio.h>
 #endif
+#ifdef HAVE_POLL_H
+/* Use poll instead select */
+#include <poll.h>
+#endif
 
 #if defined(WIN32) || (defined(NETWARE) && defined(__NOVELL_LIBC__))
 #define JK_IS_SOCKET_ERROR(x) ((x) == SOCKET_ERROR)
@@ -423,6 +427,7 @@ jk_sock_t jk_open_socket(struct sockaddr_in *addr, int keepalive,
     jk_sock_t sd;
     int set = 1;
     int ret = 0;
+    int flags = 0;
 #ifdef SO_LINGER
     struct linger li;
 #endif
@@ -430,7 +435,10 @@ jk_sock_t jk_open_socket(struct sockaddr_in *addr, int keepalive,
     JK_TRACE_ENTER(l);
 
     errno = 0;
-    sd = socket(AF_INET, SOCK_STREAM, 0);
+#if defined(SOCK_CLOEXEC) && defined(USE_SOCK_CLOEXEC)
+    flags |= SOCK_CLOEXEC;
+#endif
+    sd = socket(AF_INET, SOCK_STREAM | flags, 0);
     if (!IS_VALID_SOCKET(sd)) {
         JK_GET_SOCKET_ERRNO();
         jk_log(l, JK_LOG_ERROR,
@@ -438,6 +446,26 @@ jk_sock_t jk_open_socket(struct sockaddr_in *addr, int keepalive,
         JK_TRACE_EXIT(l);
         return JK_INVALID_SOCKET;
     }
+#if defined(FD_CLOEXEC) && !defined(USE_SOCK_CLOEXEC)
+    if ((flags = fcntl(sd, F_GETFD)) == -1) {
+        JK_GET_SOCKET_ERRNO();
+        jk_log(l, JK_LOG_ERROR,
+               "fcntl() failed (errno=%d)", errno);
+        jk_close_socket(sd, l);
+        JK_TRACE_EXIT(l);
+        return JK_INVALID_SOCKET;
+    }
+    flags |= FD_CLOEXEC;
+    if (fcntl(sd, F_SETFD, flags) == -1) {
+        JK_GET_SOCKET_ERRNO();
+        jk_log(l, JK_LOG_ERROR,
+               "fcntl() failed (errno=%d)", errno);
+        jk_close_socket(sd, l);
+        JK_TRACE_EXIT(l);
+        return JK_INVALID_SOCKET;
+    }
+#endif
+
     /* Disable Nagle algorithm */
     if (setsockopt(sd, IPPROTO_TCP, TCP_NODELAY, (SET_TYPE)&set,
                    sizeof(set))) {
@@ -630,10 +658,12 @@ int jk_close_socket(jk_sock_t sd, jk_logger_t *l)
 #if defined(WIN32) || (defined(NETWARE) && defined(__NOVELL_LIBC__))
     rc = closesocket(sd) ? -1 : 0;
 #else
-    rc = close(sd);
+    do {
+        rc = close(sd);
+    } while (JK_IS_SOCKET_ERROR(rc) && (errno == EINTR || errno == EAGAIN));
 #endif
-    errno = save_errno;
     JK_TRACE_EXIT(l);
+    errno = save_errno;
     return rc;
 }
 
@@ -650,6 +680,14 @@ int jk_close_socket(jk_sock_t sd, jk_logger_t *l)
 #endif
 #endif
 
+#ifndef SHUT_RD
+#ifdef SD_RECEIVE
+#define SHUT_RD SD_RECEIVE
+#else
+#define SHUT_RD 0x00
+#endif
+#endif
+
 /** Drain and close the socket
  * @param sd        socket to close
  * @param l         logger
@@ -664,9 +702,12 @@ int jk_shutdown_socket(jk_sock_t sd, jk_logger_t *l)
     char dummy[512];
     int rc = 0;
     int rd = 0;
+    int rp = 0;
     int save_errno;
-    fd_set rs;
+    int timeout = SECONDS_TO_LINGER * 1000;
+#ifndef HAVE_POLL
     struct timeval tv;
+#endif
     time_t start = time(NULL);
 
     JK_TRACE_ENTER(l);
@@ -692,10 +733,16 @@ int jk_shutdown_socket(jk_sock_t sd, jk_logger_t *l)
         return rc;
     }
 
-    /* Set up to wait for readable data on socket... */
-    FD_ZERO(&rs);
-
     do {
+#ifdef HAVE_POLL
+        struct pollfd fds;
+
+        fds.fd = sd;
+        fds.events = POLLIN;
+#else
+        fd_set rs;
+
+        FD_ZERO(&rs);
         /* Read all data from the peer until we reach "end-of-file"
          * (FIN from peer) or we've exceeded our overall timeout. If the
          * backend does not send us bytes within 2 seconds
@@ -703,10 +750,16 @@ int jk_shutdown_socket(jk_sock_t sd, jk_logger_t *l)
          * close the connection.
          */
         FD_SET(sd, &rs);
-        tv.tv_sec  = SECONDS_TO_LINGER;
-        tv.tv_usec = 0;
-
-        if (select((int)sd + 1, &rs, NULL, NULL, &tv) > 0) {
+        tv.tv_sec  = timeout / 1000;
+        tv.tv_usec = (timeout % 1000) * 1000;
+#endif
+        rp = 0;
+#ifdef HAVE_POLL
+        if (poll(&fds, 1, timeout) > 0)
+#else
+        if (select((int)sd + 1, &rs, NULL, NULL, &tv) > 0)
+#endif
+        {
             do {
 #if defined(WIN32) || (defined(NETWARE) && defined(__NOVELL_LIBC__))
                 rc = recv(sd, &dummy[0], sizeof(dummy), 0);
@@ -716,21 +769,50 @@ int jk_shutdown_socket(jk_sock_t sd, jk_logger_t *l)
                 rc = read(sd, &dummy[0], sizeof(dummy));
 #endif
                 if (rc > 0)
-                    rd += rc;
+                    rp += rc;
             } while (JK_IS_SOCKET_ERROR(rc) && (errno == EINTR || errno == EAGAIN));
 
-            if (rc <= 0)
+            if (rc < 0)
                 break;
         }
         else
             break;
-
+        rd += rp;
+        if (rp < sizeof(dummy)) {
+            if (timeout > SECONDS_TO_LINGER) {
+                /* Try once again with 1000 times smaller timeout
+                 * In our case 2 msec.
+                 */
+                timeout = SECONDS_TO_LINGER;
+                continue;
+            }
+            /* We have read less then size of buffer
+             * It's a good chance there will be no more data
+             * to read.
+             */
+            if ((rc = sononblock(sd))) {
+                rc = jk_close_socket(sd, l);
+                if (JK_IS_DEBUG_LEVEL(l))
+                    jk_log(l, JK_LOG_DEBUG,
+                           "error setting socket %d to nonblocking", sd);
+                errno = save_errno;
+                JK_TRACE_EXIT(l);
+                return rc;
+            }
+            if (JK_IS_DEBUG_LEVEL(l))
+                jk_log(l, JK_LOG_DEBUG,
+                       "shutting down the read side of socket %d", sd);
+            shutdown(sd, SHUT_RD);
+            break;
+        }
+        timeout = SECONDS_TO_LINGER * 1000;
     } while (difftime(time(NULL), start) < MAX_SECS_TO_LINGER);
 
     rc = jk_close_socket(sd, l);
     if (JK_IS_DEBUG_LEVEL(l))
-        jk_log(l, JK_LOG_DEBUG, "Shutdown socket %d and read %d lingering bytes",
-               sd, rd);
+        jk_log(l, JK_LOG_DEBUG,
+               "Shutdown socket %d and read %d lingering bytes in %d sec.",
+               sd, rd, (int)difftime(time(NULL), start));
     errno = save_errno;
     JK_TRACE_EXIT(l);
     return rc;
@@ -820,9 +902,9 @@ int jk_tcp_socket_recvfull(jk_sock_t sd, unsigned char *b, int len, jk_logger_t
         } while (JK_IS_SOCKET_ERROR(rd) && errno == EINTR);
 
         if (JK_IS_SOCKET_ERROR(rd)) {
+            rd = (errno > 0) ? -errno : errno;
             jk_shutdown_socket(sd, l);
             JK_TRACE_EXIT(l);
-            rd = (errno > 0) ? -errno : errno;
             return (rd == 0) ? JK_SOCKET_EOF : rd;
         }
         else if (rd == 0) {
@@ -864,6 +946,56 @@ char *jk_dump_hinfo(struct sockaddr_in *saddr, char *buf)
  *                to allow for iterative waiting
  * @remark        Cares about errno
  */
+#ifdef HAVE_POLL
+int jk_is_input_event(jk_sock_t sd, int timeout, jk_logger_t *l)
+{
+    struct pollfd fds;
+    int rc;
+    int save_errno;
+
+    JK_TRACE_ENTER(l);
+
+    errno = 0;
+    fds.fd = sd;
+    fds.events = POLLIN;
+    fds.revents = 0;
+
+    do {
+        rc = poll(&fds, 1, timeout);
+    } while (rc < 0 && errno == EINTR);
+
+    if (rc == 0) {
+        /* Timeout. Set the errno to timeout */
+        errno = ETIMEDOUT;
+        JK_TRACE_EXIT(l);
+        return JK_FALSE;
+    }
+    else if (rc < 0) {
+        save_errno = errno;
+        if (JK_IS_DEBUG_LEVEL(l)) {
+            jk_log(l, JK_LOG_DEBUG,
+                   "error during poll on socket sd = %d (errno=%d)", sd, errno);
+        }
+        errno = save_errno;
+        JK_TRACE_EXIT(l);
+        return JK_FALSE;
+    }
+    if ((fds.revents & (POLLERR | POLLHUP))) {
+        save_errno = fds.revents & (POLLERR | POLLHUP);
+        if (JK_IS_DEBUG_LEVEL(l)) {
+            jk_log(l, JK_LOG_DEBUG,
+                   "error event during poll on socket sd = %d (event=%d)",
+               sd, save_errno);
+        }
+        errno = save_errno;
+        JK_TRACE_EXIT(l);
+        return JK_FALSE;        
+    }
+    errno = 0;
+    JK_TRACE_EXIT(l);
+    return JK_TRUE;
+}
+#else
 int jk_is_input_event(jk_sock_t sd, int timeout, jk_logger_t *l)
 {
     fd_set rset;
@@ -895,8 +1027,11 @@ int jk_is_input_event(jk_sock_t sd, int timeout, jk_logger_t *l)
     }
     else if (rc < 0) {
         save_errno = errno;
-        jk_log(l, JK_LOG_WARNING,
-               "error during select on socket sd = %d (errno=%d)", sd, errno);
+        if (JK_IS_DEBUG_LEVEL(l)) {
+            jk_log(l, JK_LOG_DEBUG,
+                   "error during select on socket sd = %d (errno=%d)",
+                   sd, errno);
+        }
         errno = save_errno;
         JK_TRACE_EXIT(l);
         return JK_FALSE;
@@ -905,6 +1040,7 @@ int jk_is_input_event(jk_sock_t sd, int timeout, jk_logger_t *l)
     JK_TRACE_EXIT(l);
     return JK_TRUE;
 }
+#endif
 
 /** Test if a socket is still connected
  * @param sd   socket to use
@@ -914,6 +1050,45 @@ int jk_is_input_event(jk_sock_t sd, int timeout, jk_logger_t *l)
  * @remark     Always closes socket in case of error
  * @remark     Cares about errno
  */
+#ifdef HAVE_POLL
+int jk_is_socket_connected(jk_sock_t sd, jk_logger_t *l)
+{
+    struct pollfd fds;
+    int rc;
+
+    JK_TRACE_ENTER(l);
+
+    errno = 0;
+    fds.fd = sd;
+    fds.events = POLLIN;
+
+    do {
+        rc = poll(&fds, 1, 0);
+    } while (rc < 0 && errno == EINTR);
+
+    if (rc == 0) {
+        /* If we get a timeout, then we are still connected */
+        JK_TRACE_EXIT(l);
+        return JK_TRUE;
+    }
+    else if (rc == 1 && fds.revents == POLLIN) {
+        char buf;
+        do {
+            rc = (int)recvfrom(sd, &buf, 1, MSG_PEEK, NULL, NULL);
+        } while (rc < 0 && errno == EINTR);
+        if (rc == 1) {
+            /* There is at least one byte to read. */
+            JK_TRACE_EXIT(l);
+            return JK_TRUE;
+        }
+    }
+    jk_shutdown_socket(sd, l);
+
+    JK_TRACE_EXIT(l);
+    return JK_FALSE;
+}
+
+#else
 int jk_is_socket_connected(jk_sock_t sd, jk_logger_t *l)
 {
     fd_set fd;
@@ -970,3 +1145,5 @@ int jk_is_socket_connected(jk_sock_t sd, jk_logger_t *l)
     JK_TRACE_EXIT(l);
     return JK_FALSE;
 }
+#endif
+
