@@ -18,7 +18,7 @@
 /***************************************************************************
  * Description: Socket connections header file                             *
  * Author:      Gal Shachor <shachor@il.ibm.com>                           *
- * Version:     $Revision: 466585 $                                               *
+ * Version:     $Revision: 706039 $                                               *
  ***************************************************************************/
 
 #ifndef JK_CONNECT_H
@@ -37,24 +37,37 @@ extern "C"
 #endif                          /* __cplusplus */
 
 #define JK_SOCKET_EOF      (-2)
-#define JK_SOCKET_ERROR    (-3)
 
-int jk_resolve(const char *host, int port, struct sockaddr_in *rc);
+int jk_resolve(const char *host, int port, struct sockaddr_in *rc, void *pool, jk_logger_t *l);
 
 jk_sock_t jk_open_socket(struct sockaddr_in *addr, int keepalive,
-                         int timeout, int sock_buf, jk_logger_t *l);
+                         int timeout, int connect_timeout,
+                         int sock_buf, jk_logger_t *l);
 
-int jk_close_socket(jk_sock_t s);
+int jk_close_socket(jk_sock_t sd, jk_logger_t *l);
 
-int jk_shutdown_socket(jk_sock_t s);
+int jk_shutdown_socket(jk_sock_t sd, jk_logger_t *l);
 
-int jk_tcp_socket_sendfull(jk_sock_t sd, const unsigned char *b, int len);
+int jk_tcp_socket_sendfull(jk_sock_t sd, const unsigned char *b, int len, jk_logger_t *l);
 
-int jk_tcp_socket_recvfull(jk_sock_t sd, unsigned char *b, int len);
+int jk_tcp_socket_recvfull(jk_sock_t sd, unsigned char *b, int len, jk_logger_t *l);
 
 char *jk_dump_hinfo(struct sockaddr_in *saddr, char *buf);
 
-int jk_is_socket_connected(jk_sock_t sd);
+int jk_is_input_event(jk_sock_t sd, int timeout, jk_logger_t *l);
+
+int jk_is_socket_connected(jk_sock_t sd, jk_logger_t *l);
+
+
+/***
+ * i5/OS V5R4 need ASCII<->EBCDIC translation for inet_addr() call
+ */
+#if !defined(AS400_UTF8)
+
+#define jk_inet_addr inet_addr
+
+#endif
+
 
 #ifdef __cplusplus
 }
