@@ -22,7 +22,7 @@
  * Author:      Mladen Turk <mturk@apache.org>                             *
  * Author:      Rainer Jung <rjung@apache.org>                             *
  * Based on:                                                               *
- * Version:     $Revision: 756452 $                                          *
+ * Version:     $Revision: 915149 $                                          *
  ***************************************************************************/
 
 #include "jk_pool.h"
@@ -840,7 +840,11 @@ static int find_bysession_route(jk_ws_service_t *s,
             }
             else if (*wr.domain && !uses_domain) {
                 candidate = find_best_bydomain(s, p, wr.domain, states, l);
-                s->route = wr.domain;
+                if (candidate >= 0) {
+                    s->route = wr.domain;
+                } else {
+                    s->route = NULL;
+                }
             }
             if (candidate >= 0) {
                 wr = p->lb_workers[candidate];
@@ -1525,7 +1529,7 @@ static int JK_METHOD validate(jk_worker_t *pThis,
                 JK_TRACE_EXIT(l);
                 return JK_FALSE;
             }
-
+            memset(p->lb_workers, 0, num_of_workers * sizeof(lb_sub_worker_t));
             for (i = 0; i < num_of_workers; i++) {
                 p->lb_workers[i].s = jk_shm_alloc_lb_sub_worker(&p->p);
                 if (p->lb_workers[i].s == NULL) {
