@@ -19,7 +19,7 @@
  * Description: General purpose map object                                 *
  * Author:      Gal Shachor <shachor@il.ibm.com>                           *
  * Author:      Mladen Turk <mturk@apache.org>                             *
- * Version:     $Revision: 591502 $                                          *
+ * Version:     $Revision: 756058 $                                          *
  ***************************************************************************/
 #if defined(AS400) && !defined(AS400_UTF8)
 #include "apr_xlate.h"
@@ -80,6 +80,7 @@ struct jk_map
 static void trim_prp_comment(char *prp);
 static size_t trim(char *s);
 static int map_realloc(jk_map_t *m);
+static char *jk_map_replace_properties(jk_map_t *m, jk_map_t *env, char *value);
 
 int jk_map_alloc(jk_map_t **m)
 {
@@ -249,7 +250,7 @@ char **jk_map_get_string_list(jk_map_t *m,
     const char *l = jk_map_get_string(m, name, def);
     char **ar = NULL;
 
-#ifdef _REENTRANT
+#ifdef _MT_CODE_PTHREAD
     char *lasts;
 #endif
 
@@ -269,7 +270,7 @@ char **jk_map_get_string_list(jk_map_t *m,
          * GS, in addition to VG's patch, we now need to
          * strtok also by a "*"
          */
-#ifdef _REENTRANT
+#ifdef _MT_CODE_PTHREAD
         for (p = strtok_r(v, " \t,", &lasts);
              p; p = strtok_r(NULL, " \t,", &lasts))
 #else
@@ -305,7 +306,7 @@ int jk_map_get_int_list(jk_map_t *m,
 {
     const char *l = jk_map_get_string(m, name, def);
 
-#ifdef _REENTRANT
+#ifdef _MT_CODE_PTHREAD
     char *lasts;
 #endif
 
@@ -326,7 +327,7 @@ int jk_map_get_int_list(jk_map_t *m,
          * GS, in addition to VG's patch, we now need to
          * strtok also by a "*"
          */
-#ifdef _REENTRANT
+#ifdef _MT_CODE_PTHREAD
         for (p = strtok_r(v, " \t,", &lasts);
              p; p = strtok_r(NULL, " \t,", &lasts))
 #else
@@ -454,7 +455,7 @@ static int jk_map_handle_duplicates(jk_map_t *m, const char *prp, char **v,
     }
 }
 
-int jk_map_read_property(jk_map_t *m, const char *str,
+int jk_map_read_property(jk_map_t *m, jk_map_t *env, const char *str,
                          int treatment, jk_logger_t *l)
 {
     int rc = JK_TRUE;
@@ -474,16 +475,14 @@ int jk_map_read_property(jk_map_t *m, const char *str,
         if (v) {
             *v = '\0';
             v++;
-            trim(prp);
-            trim(v);
-            if (strlen(v) && strlen(prp)) {
+            if (trim(v) && trim(prp)) {
                 if (treatment == JK_MAP_HANDLE_RAW) {
                     v = jk_pool_strdup(&m->p, v);
                 }
                 else {
                     if (jk_map_validate_property(prp, l) == JK_FALSE)
                         return JK_FALSE;
-                    v = jk_map_replace_properties(m, v);
+                    v = jk_map_replace_properties(m, env, v);
                     if (jk_map_handle_duplicates(m, prp, &v, treatment, l) == JK_TRUE)
                         v = jk_pool_strdup(&m->p, v);
                 }
@@ -505,7 +504,7 @@ int jk_map_read_property(jk_map_t *m, const char *str,
 }
 
 
-int jk_map_read_properties(jk_map_t *m, const char *f, time_t *modified,
+int jk_map_read_properties(jk_map_t *m, jk_map_t *env, const char *f, time_t *modified,
                            int treatment, jk_logger_t *l)
 {
     int rc = JK_FALSE;
@@ -530,7 +529,7 @@ int jk_map_read_properties(jk_map_t *m, const char *f, time_t *modified,
             while (NULL != (prp = fgets(buf, LENGTH_OF_LINE, fp))) {
                 trim_prp_comment(prp);
                 if (*prp) {
-                    if ((rc = jk_map_read_property(m, prp, treatment, l)) == JK_FALSE)
+                    if ((rc = jk_map_read_property(m, env, prp, treatment, l)) == JK_FALSE)
                         break;
                 }
             }
@@ -688,9 +687,9 @@ static int map_realloc(jk_map_t *m)
  *  Replace $(property) in value.
  *
  */
-char *jk_map_replace_properties(jk_map_t *m, const char *value)
+static char *jk_map_replace_properties(jk_map_t *m, jk_map_t *env, char *value)
 {
-    char *rc = (char *)value;
+    char *rc = value;
     char *env_start = rc;
     int rec = 0;
 
@@ -712,6 +711,11 @@ char *jk_map_replace_properties(jk_map_t *m, const char *value)
             if (!env_value) {
                 env_value = getenv(env_name);
             }
+            if (!env_value && env) {
+                /* Search inside local environment table */
+                env_value = jk_map_get_string(env, env_name, NULL);
+            }
+
 #if defined(WIN32)
             if (!env_value) {
                 /* Try the env block from calling process */
@@ -809,7 +813,6 @@ int jk_map_resolve_references(jk_map_t *m, const char *prefix,
                             if (rc == JK_FALSE) {
                                 break;
                             }
-                            *v = '\0';
                         }
                     }
                 }
