@@ -18,7 +18,7 @@
 /***************************************************************************
  * Description: Map object header file                                     *
  * Author:      Gal Shachor <shachor@il.ibm.com>                           *
- * Version:     $Revision: 591502 $                                           *
+ * Version:     $Revision: 756058 $                                           *
  ***************************************************************************/
 
 #ifndef JK_MAP_H
@@ -70,9 +70,9 @@ int jk_map_add(jk_map_t *m, const char *name, const void *value);
 
 int jk_map_put(jk_map_t *m, const char *name, const void *value, void **old);
 
-int jk_map_read_property(jk_map_t *m, const char *str, int treatment, jk_logger_t *l);
+int jk_map_read_property(jk_map_t *m, jk_map_t *env, const char *str, int treatment, jk_logger_t *l);
 
-int jk_map_read_properties(jk_map_t *m, const char *f, time_t *modified, int treatment, jk_logger_t *l);
+int jk_map_read_properties(jk_map_t *m, jk_map_t *env, const char *f, time_t *modified, int treatment, jk_logger_t *l);
 
 int jk_map_size(jk_map_t *m);
 
@@ -84,12 +84,6 @@ void jk_map_dump(jk_map_t *m, jk_logger_t *l);
 
 int jk_map_copy(jk_map_t *src, jk_map_t *dst);
 
-/**
- *  Replace $(property) in value.
- *
- */
-char *jk_map_replace_properties(jk_map_t *m, const char *value);
-
 int jk_map_resolve_references(jk_map_t *m, const char *prefix, int wildcard, int depth, jk_logger_t *l);
 
 int jk_map_inherit_properties(jk_map_t *m, const char *from, const char *to, jk_logger_t *l);
