@@ -20,7 +20,7 @@
  *                                                                         *
  * Author:      Gal Shachor <shachor@il.ibm.com>                           *
  * Author:      Mladen Turk <mturk@apache.org>                             *
- * Version:     $Revision: 605846 $                                          *
+ * Version:     $Revision: 890739 $                                          *
  ***************************************************************************/
 
 #include "jk_pool.h"
@@ -28,6 +28,8 @@
 #include "jk_map.h"
 #include "jk_mt.h"
 #include "jk_uri_worker_map.h"
+#include "jk_worker.h"
+#include "jk_lb_worker.h"
 
 #ifdef WIN32
 #define JK_STRCMP   strcasecmp
@@ -37,6 +39,18 @@
 #define JK_STRNCMP  strncmp
 #endif
 
+#define JK_UWMAP_EXTENSION_REPLY_TIMEOUT  "reply_timeout="
+#define JK_UWMAP_EXTENSION_ACTIVE         "active="
+#define JK_UWMAP_EXTENSION_DISABLED       "disabled="
+#define JK_UWMAP_EXTENSION_STOPPED        "stopped="
+#define JK_UWMAP_EXTENSION_FAIL_ON_STATUS "fail_on_status="
+#define JK_UWMAP_EXTENSION_USE_SRV_ERRORS "use_server_errors="
+
+#define IND_SWITCH(x)                      (((x)+1) % 2)
+#define IND_THIS(x)                        ((x)[uw_map->index])
+#define IND_NEXT(x)                        ((x)[IND_SWITCH(uw_map->index)])
+
+#define STRNULL_FOR_NULL(x) ((x) ? (x) : "(null)")
 
 static const char *uri_worker_map_source_type[] = {
     "unknown",
@@ -140,45 +154,53 @@ static void worker_qsort(jk_uri_worker_map_t *uw_map)
 {
 
    /* Sort remaining args using Quicksort algorithm: */
-   qsort((void *)uw_map->maps, uw_map->size,
+   qsort((void *)IND_NEXT(uw_map->maps), IND_NEXT(uw_map->size),
          sizeof(uri_worker_record_t *), worker_compare );
 
 }
 
-/* Match = 0, NoMatch = 1, Abort = -1
- * Based loosely on sections of wildmat.c by Rich Salz
- */
-static int wildchar_match(const char *str, const char *exp, int icase)
+/* Dump the map contents - only call if debug log is active. */
+static void uri_worker_map_dump(jk_uri_worker_map_t *uw_map,
+                                const char *reason, jk_logger_t *l)
 {
-    int x, y;
-
-    for (x = 0, y = 0; exp[y]; ++y, ++x) {
-        if (!str[x] && exp[y] != '*')
-            return -1;
-        if (exp[y] == '*') {
-            while (exp[++y] == '*');
-            if (!exp[y])
-                return 0;
-            while (str[x]) {
-                int ret;
-                if ((ret = wildchar_match(&str[x++], &exp[y], icase)) != 1)
-                    return ret;
-            }
-            return -1;
+    JK_TRACE_ENTER(l);
+    if (uw_map) {
+        int i, off, k;
+        uri_worker_record_t *uwr = NULL;
+        char buf[32];
+        jk_log(l, JK_LOG_DEBUG, "uri map dump %s: index=%d file='%s' reject_unsafe=%d "
+               "reload=%d modified=%d checked=%d",
+               reason, uw_map->index, STRNULL_FOR_NULL(uw_map->fname), uw_map->reject_unsafe,
+               uw_map->reload, uw_map->modified, uw_map->checked);
+        for(i=0;i<=1;i++) {
+            jk_log(l, JK_LOG_DEBUG, "generation %d: size=%d nosize=%d capacity=%d",
+                   i, uw_map->size[i], uw_map->nosize[i], uw_map->capacity[i], uw_map->maps[i]);
         }
-        else if (exp[y] != '?') {
-            if (icase && (tolower(str[x]) != tolower(exp[y])))
-                return 1;
-            else if (!icase && str[x] != exp[y])
-                return 1;
+
+        off = uw_map->index;
+        for(i=0;i<=1;i++) {
+            unsigned int j;
+            k = (i + off) % 2;
+            for (j = 0; j < uw_map->size[k]; j++) {
+                uwr = uw_map->maps[k][j];
+                jk_log(l, JK_LOG_DEBUG, "%s (%d) map #%d: uri=%s worker=%s context=%s "
+                       "source=%s type=%s len=%d",
+                       i ? "NEXT" : "THIS", i, j,
+                       STRNULL_FOR_NULL(uwr->uri), STRNULL_FOR_NULL(uwr->worker_name),
+                       STRNULL_FOR_NULL(uwr->context), STRNULL_FOR_NULL(uri_worker_map_get_source(uwr,l)),
+                       STRNULL_FOR_NULL(uri_worker_map_get_match(uwr,buf,l)), uwr->context_len);
+            }
         }
     }
-    return (str[x] != '\0');
+    JK_TRACE_EXIT(l);
 }
 
+
 int uri_worker_map_alloc(jk_uri_worker_map_t **uw_map_p,
                          jk_map_t *init_data, jk_logger_t *l)
 {
+    int i;
+
     JK_TRACE_ENTER(l);
 
     if (uw_map_p) {
@@ -198,10 +220,15 @@ int uri_worker_map_alloc(jk_uri_worker_map_t **uw_map_p,
 
         jk_open_pool(&(uw_map->p),
                      uw_map->buf, sizeof(jk_pool_atom_t) * BIG_POOL_SIZE);
-        uw_map->size = 0;
-        uw_map->nosize = 0;
-        uw_map->capacity = 0;
-        uw_map->maps = NULL;
+        for(i=0;i<=1;i++) {
+            jk_open_pool(&(uw_map->p_dyn[i]),
+                         uw_map->buf_dyn[i], sizeof(jk_pool_atom_t) * BIG_POOL_SIZE);
+            uw_map->size[i] = 0;
+            uw_map->nosize[i] = 0;
+            uw_map->capacity[i] = 0;
+            uw_map->maps[i] = NULL;
+        }
+        uw_map->index = 0;
         uw_map->fname = NULL;
         uw_map->reject_unsafe = 0;
         uw_map->reload = JK_URIMAP_DEF_RELOAD;
@@ -227,6 +254,8 @@ static int uri_worker_map_close(jk_uri_worker_map_t *uw_map, jk_logger_t *l)
     if (uw_map) {
         int i;
         JK_DELETE_CS(&(uw_map->cs), i);
+        jk_close_pool(&uw_map->p_dyn[0]);
+        jk_close_pool(&uw_map->p_dyn[1]);
         jk_close_pool(&uw_map->p);
         JK_TRACE_EXIT(l);
         return JK_TRUE;
@@ -263,24 +292,24 @@ int uri_worker_map_free(jk_uri_worker_map_t **uw_map, jk_logger_t *l)
 
 static int uri_worker_map_realloc(jk_uri_worker_map_t *uw_map)
 {
-    if (uw_map->size == uw_map->capacity) {
+    if (IND_NEXT(uw_map->size) == IND_NEXT(uw_map->capacity)) {
         uri_worker_record_t **uwr;
-        int capacity = uw_map->capacity + UW_INC_SIZE;
+        int capacity = IND_NEXT(uw_map->capacity) + UW_INC_SIZE;
 
         uwr =
-            (uri_worker_record_t **) jk_pool_alloc(&uw_map->p,
+            (uri_worker_record_t **) jk_pool_alloc(&IND_NEXT(uw_map->p_dyn),
                                                    sizeof(uri_worker_record_t
                                                           *) * capacity);
 
         if (!uwr)
             return JK_FALSE;
 
-        if (uw_map->capacity && uw_map->maps)
-            memcpy(uwr, uw_map->maps,
-                   sizeof(uri_worker_record_t *) * uw_map->capacity);
+        if (IND_NEXT(uw_map->capacity) && IND_NEXT(uw_map->maps))
+            memcpy(uwr, IND_NEXT(uw_map->maps),
+                   sizeof(uri_worker_record_t *) * IND_NEXT(uw_map->capacity));
 
-        uw_map->maps = uwr;
-        uw_map->capacity = capacity;
+        IND_NEXT(uw_map->maps) = uwr;
+        IND_NEXT(uw_map->capacity) = capacity;
     }
 
     return JK_TRUE;
@@ -292,39 +321,271 @@ static int uri_worker_map_realloc(jk_uri_worker_map_t *uw_map)
  */
 
 static int uri_worker_map_clear(jk_uri_worker_map_t *uw_map,
-                         unsigned int source_type, jk_logger_t *l)
+                                jk_logger_t *l)
 {
     uri_worker_record_t *uwr = NULL;
     unsigned int i;
-    unsigned int j;
+    unsigned int new_size = 0;
+    unsigned int new_nosize = 0;
 
     JK_TRACE_ENTER(l);
 
-    for (i = 0; i < uw_map->size; i++) {
-        uwr = uw_map->maps[i];
-        if (uwr->source_type == source_type) {
-            jk_log(l, JK_LOG_DEBUG,
-                   "deleting map rule '%s=%s' source '%s'",
-                   uwr->context, uwr->worker_name, uri_worker_map_get_source(uwr, l));
-            for (j = i; j < uw_map->size-1; j++)
-                uw_map->maps[j] = uw_map->maps[j+1];
-            uw_map->size--;
-            i--;
+    IND_NEXT(uw_map->maps) =
+            (uri_worker_record_t **) jk_pool_alloc(&(IND_NEXT(uw_map->p_dyn)),
+                                                   sizeof(uri_worker_record_t
+                                                          *) * IND_THIS(uw_map->size));
+    IND_NEXT(uw_map->capacity) = IND_THIS(uw_map->size);
+    IND_NEXT(uw_map->size) = 0;
+    IND_NEXT(uw_map->nosize) = 0;
+    for (i = 0; i < IND_THIS(uw_map->size); i++) {
+        uwr = IND_THIS(uw_map->maps)[i];
+        if (uwr->source_type == SOURCE_TYPE_URIMAP) {
+            if (JK_IS_DEBUG_LEVEL(l))
+                jk_log(l, JK_LOG_DEBUG,
+                       "deleting map rule '%s=%s' source '%s'",
+                       uwr->context, uwr->worker_name, uri_worker_map_get_source(uwr, l));
+        }
+        else {
+            IND_NEXT(uw_map->maps)[new_size] = uwr;
+            new_size++;
+            if (uwr->match_type & MATCH_TYPE_NO_MATCH)
+                new_nosize++;
         }
     }
+    IND_NEXT(uw_map->size) = new_size;
+    IND_NEXT(uw_map->nosize) = new_nosize;
 
     JK_TRACE_EXIT(l);
     return JK_TRUE;
 }
 
+static void extract_activation(jk_uri_worker_map_t *uw_map,
+                               uri_worker_record_t *uwr,
+                               lb_worker_t *lb,
+                               int *activations,
+                               char *workers,
+                               int activation,
+                               jk_logger_t *l)
+{
+    unsigned int i;
+    jk_pool_t *p;
+    char *worker;
+#ifdef _MT_CODE_PTHREAD
+    char *lasts;
+#endif
+
+    JK_TRACE_ENTER(l);
+
+    if (uwr->source_type == SOURCE_TYPE_URIMAP)
+        p = &IND_NEXT(uw_map->p_dyn);
+    else
+        p = &uw_map->p;
+    worker = jk_pool_strdup(p, workers);
+
+#ifdef _MT_CODE_PTHREAD
+    for (worker = strtok_r(worker, ", ", &lasts);
+         worker; worker = strtok_r(NULL, ", ", &lasts)) {
+#else
+    for (worker = strtok(worker, ", "); worker; worker = strtok(NULL, ", ")) {
+#endif
+        for (i=0; i<lb->num_of_workers; i++) {
+            if (!strcmp(worker, lb->lb_workers[i].name)) {
+                if (activations[i] != JK_LB_ACTIVATION_UNSET)
+                    jk_log(l, JK_LOG_WARNING,
+                           "inconsistent activation overwrite for member %s "
+                           "of load balancer %s: '%s' replaced by '%s'",
+                           worker, lb->name,
+                           jk_lb_get_activation_direct(activations[i], l),
+                           jk_lb_get_activation_direct(activation, l));
+                activations[i] = activation;
+                break;
+            }
+        }
+        if (i >= lb->num_of_workers)
+            jk_log(l, JK_LOG_WARNING,
+                   "could not find member %s of load balancer %s",
+                   worker, lb->name);
+    }
+
+    JK_TRACE_EXIT(l);
+
+}
+
+static void extract_fail_on_status(jk_uri_worker_map_t *uw_map,
+                                   uri_worker_record_t *uwr,
+                                   jk_logger_t *l)
+{
+    unsigned int i;
+    int j;
+    int cnt = 1;
+    jk_pool_t *p;
+    char *status;
+#ifdef _MT_CODE_PTHREAD
+    char *lasts;
+#endif
+
+    JK_TRACE_ENTER(l);
+
+    for (i=0; i<strlen(uwr->extensions.fail_on_status_str); i++) {
+        if (uwr->extensions.fail_on_status_str[i] == ',' ||
+            uwr->extensions.fail_on_status_str[i] == ' ')
+            cnt++;
+    }
+    uwr->extensions.fail_on_status_size = cnt;
+
+    if (uwr->source_type == SOURCE_TYPE_URIMAP)
+        p = &IND_NEXT(uw_map->p_dyn);
+    else
+        p = &uw_map->p;
+    status = jk_pool_strdup(p, uwr->extensions.fail_on_status_str);
+    uwr->extensions.fail_on_status = (int *)jk_pool_alloc(p,
+                                            uwr->extensions.fail_on_status_size * sizeof(int));
+    if (!uwr->extensions.fail_on_status) {
+        jk_log(l, JK_LOG_ERROR,
+               "can't alloc extensions fail_on_status list");
+        JK_TRACE_EXIT(l);
+        return;
+    } else if (JK_IS_DEBUG_LEVEL(l))
+        jk_log(l, JK_LOG_DEBUG,
+               "Allocated fail_on_status array of size %d for worker %s",
+               uwr->extensions.fail_on_status_size, uwr->worker_name);
 
 
+    for (j=0; j<uwr->extensions.fail_on_status_size; j++) {
+        uwr->extensions.fail_on_status[j] = 0;
+    }
+
+    cnt = 0;
+#ifdef _MT_CODE_PTHREAD
+    for (status = strtok_r(status, ", ", &lasts);
+         status; status = strtok_r(NULL, ", ", &lasts)) {
+#else
+    for (status = strtok(status, ", "); status; status = strtok(NULL, ", ")) {
+#endif
+        uwr->extensions.fail_on_status[cnt] = atoi(status);
+        cnt++;
+    }
+
+    JK_TRACE_EXIT(l);
+
+}
+
+void uri_worker_map_switch(jk_uri_worker_map_t *uw_map, jk_logger_t *l)
+{
+    int new_index;
+
+    JK_TRACE_ENTER(l);
+
+    if (uw_map) {
+        new_index = IND_SWITCH(uw_map->index);
+        if (JK_IS_DEBUG_LEVEL(l))
+            jk_log(l, JK_LOG_DEBUG,
+                   "Switching uri worker map from index %d to index %d",
+                   uw_map->index, new_index);
+        uw_map->index = new_index;
+        jk_reset_pool(&(IND_NEXT(uw_map->p_dyn)));
+    }
+
+    JK_TRACE_EXIT(l);
+
+}
+
+void uri_worker_map_ext(jk_uri_worker_map_t *uw_map, jk_logger_t *l)
+{
+    unsigned int i;
+
+    JK_TRACE_ENTER(l);
+
+    for (i = 0; i < IND_NEXT(uw_map->size); i++) {
+        uri_worker_record_t *uwr = IND_NEXT(uw_map->maps)[i];
+        jk_worker_t *jw;
+        if(uwr->match_type & MATCH_TYPE_NO_MATCH)
+            continue;
+        jw = wc_get_worker_for_name(uwr->worker_name, l);
+        if(!jw) {
+            jk_log(l, JK_LOG_ERROR,
+                   "Could not find worker with name '%s' in uri map post processing.",
+                   uwr->worker_name);
+            continue;
+        }
+        if (JK_IS_DEBUG_LEVEL(l))
+            jk_log(l, JK_LOG_DEBUG,
+                   "Checking extension for worker %d: %s of type %s (%d)",
+                   i, uwr->worker_name, wc_get_name_for_type(jw->type,l), jw->type);
+
+        if (jw->type == JK_LB_WORKER_TYPE &&
+            (uwr->extensions.active || uwr->extensions.disabled || uwr->extensions.stopped)) {
+            int j;
+            lb_worker_t *lb = (lb_worker_t *)jw->worker_private;
+            jk_pool_t *p;
+            if (!uwr->extensions.activation) {
+                uwr->extensions.activation_size = lb->num_of_workers;
+                if (uwr->source_type == SOURCE_TYPE_URIMAP)
+                    p = &IND_NEXT(uw_map->p_dyn);
+                else
+                    p = &uw_map->p;
+                uwr->extensions.activation = (int *)jk_pool_alloc(p,
+                                                        uwr->extensions.activation_size * sizeof(int));
+                if (!uwr->extensions.activation) {
+                    jk_log(l, JK_LOG_ERROR,
+                           "can't alloc extensions activation list");
+                    continue;
+                } else if (JK_IS_DEBUG_LEVEL(l))
+                    jk_log(l, JK_LOG_DEBUG,
+                           "Allocated activations array of size %d for lb worker %s",
+                           uwr->extensions.activation_size, uwr->worker_name);
+                for (j=0; j<uwr->extensions.activation_size; j++) {
+                    uwr->extensions.activation[j] = JK_LB_ACTIVATION_UNSET;
+                }
+            }
+            if (uwr->extensions.active)
+                extract_activation(uw_map, uwr, lb, uwr->extensions.activation,
+                                   uwr->extensions.active, JK_LB_ACTIVATION_ACTIVE, l);
+            if (uwr->extensions.disabled)
+                extract_activation(uw_map, uwr, lb, uwr->extensions.activation,
+                                   uwr->extensions.disabled, JK_LB_ACTIVATION_DISABLED, l);
+            if (uwr->extensions.stopped)
+                extract_activation(uw_map, uwr, lb, uwr->extensions.activation,
+                                   uwr->extensions.stopped, JK_LB_ACTIVATION_STOPPED, l);
+        }
+        else if (uwr->extensions.active) {
+            jk_log(l, JK_LOG_WARNING,
+                   "Worker %s is not of type lb, activation extension "
+                   JK_UWMAP_EXTENSION_ACTIVE " for %s ignored",
+                   uwr->worker_name, uwr->extensions.active);
+        }
+        else if (uwr->extensions.disabled) {
+            jk_log(l, JK_LOG_WARNING,
+                   "Worker %s is not of type lb, activation extension "
+                   JK_UWMAP_EXTENSION_DISABLED " for %s ignored",
+                   uwr->worker_name, uwr->extensions.disabled);
+        }
+        else if (uwr->extensions.stopped) {
+            jk_log(l, JK_LOG_WARNING,
+                   "Worker %s is not of type lb, activation extension "
+                   JK_UWMAP_EXTENSION_STOPPED " for %s ignored",
+                   uwr->worker_name, uwr->extensions.stopped);
+        }
+        if (uwr->extensions.fail_on_status_str) {
+            extract_fail_on_status(uw_map, uwr, l);
+        }
+    }
+    if (JK_IS_DEBUG_LEVEL(l))
+        uri_worker_map_dump(uw_map, "after extension stripping", l);
+
+    JK_TRACE_EXIT(l);
+    return;
+
+}
+
+/* Add new entry to NEXT generation */
 int uri_worker_map_add(jk_uri_worker_map_t *uw_map,
                        const char *puri, const char *worker,
                        unsigned int source_type, jk_logger_t *l)
 {
     uri_worker_record_t *uwr = NULL;
     char *uri;
+    jk_pool_t *p;
     unsigned int match_type = 0;
 
     JK_TRACE_ENTER(l);
@@ -346,8 +607,12 @@ int uri_worker_map_add(jk_uri_worker_map_t *uw_map,
         JK_TRACE_EXIT(l);
         return JK_FALSE;
     }
-    uwr = (uri_worker_record_t *)jk_pool_alloc(&uw_map->p,
-                                    sizeof(uri_worker_record_t));
+    if (source_type == SOURCE_TYPE_URIMAP)
+        p = &IND_NEXT(uw_map->p_dyn);
+    else
+        p = &uw_map->p;
+
+    uwr = (uri_worker_record_t *)jk_pool_alloc(p, sizeof(uri_worker_record_t));
     if (!uwr) {
         jk_log(l, JK_LOG_ERROR,
                "can't alloc map entry");
@@ -355,7 +620,7 @@ int uri_worker_map_add(jk_uri_worker_map_t *uw_map,
         return JK_FALSE;
     }
 
-    uri = jk_pool_strdup(&uw_map->p, puri);
+    uri = jk_pool_strdup(p, puri);
     if (!uri || !worker) {
         jk_log(l, JK_LOG_ERROR,
                "can't alloc uri/worker strings");
@@ -364,11 +629,86 @@ int uri_worker_map_add(jk_uri_worker_map_t *uw_map,
     }
 
     if (*uri == '/') {
+        char *w;
+        char *param;
+#ifdef _MT_CODE_PTHREAD
+        char *lasts = NULL;
+#endif
+
+        w = jk_pool_strdup(p, worker);
+        uwr->extensions.reply_timeout = -1;
+        uwr->extensions.active = NULL;
+        uwr->extensions.disabled = NULL;
+        uwr->extensions.stopped = NULL;
+        uwr->extensions.activation_size = 0;
+        uwr->extensions.activation = NULL;
+        uwr->extensions.fail_on_status_size = 0;
+        uwr->extensions.fail_on_status = NULL;
+        uwr->extensions.fail_on_status_str = NULL;
+        uwr->extensions.use_server_error_pages = 0;
+
+#ifdef _MT_CODE_PTHREAD
+        param = strtok_r(w, ";", &lasts);
+#else
+        param = strtok(w, ";");
+#endif
+        if (param) {
+#ifdef _MT_CODE_PTHREAD
+            for (param = strtok_r(NULL, ";", &lasts); param; param = strtok_r(NULL, ";", &lasts)) {
+#else
+            for (param = strtok(NULL, ";"); param; param = strtok(NULL, ";")) {
+#endif
+                if (!strncmp(param, JK_UWMAP_EXTENSION_REPLY_TIMEOUT, strlen(JK_UWMAP_EXTENSION_REPLY_TIMEOUT))) {
+                    uwr->extensions.reply_timeout = atoi(param + strlen(JK_UWMAP_EXTENSION_REPLY_TIMEOUT));
+                }
+                else if (!strncmp(param, JK_UWMAP_EXTENSION_USE_SRV_ERRORS, strlen(JK_UWMAP_EXTENSION_USE_SRV_ERRORS))) {
+                    uwr->extensions.use_server_error_pages = atoi(param + strlen(JK_UWMAP_EXTENSION_USE_SRV_ERRORS));
+                }
+                else if (!strncmp(param, JK_UWMAP_EXTENSION_ACTIVE, strlen(JK_UWMAP_EXTENSION_ACTIVE))) {
+                    if (uwr->extensions.active)
+                        jk_log(l, JK_LOG_WARNING,
+                               "extension '%s' in uri worker map only allowed once",
+                               JK_UWMAP_EXTENSION_ACTIVE);
+                    else
+                        uwr->extensions.active = param + strlen(JK_UWMAP_EXTENSION_ACTIVE);
+                }
+                else if (!strncmp(param, JK_UWMAP_EXTENSION_DISABLED, strlen(JK_UWMAP_EXTENSION_DISABLED))) {
+                    if (uwr->extensions.disabled)
+                        jk_log(l, JK_LOG_WARNING,
+                               "extension '%s' in uri worker map only allowed once",
+                               JK_UWMAP_EXTENSION_DISABLED);
+                    else
+                        uwr->extensions.disabled = param + strlen(JK_UWMAP_EXTENSION_DISABLED);
+                }
+                else if (!strncmp(param, JK_UWMAP_EXTENSION_STOPPED, strlen(JK_UWMAP_EXTENSION_STOPPED))) {
+                    if (uwr->extensions.stopped)
+                        jk_log(l, JK_LOG_WARNING,
+                               "extension '%s' in uri worker map only allowed once",
+                               JK_UWMAP_EXTENSION_STOPPED);
+                    else
+                        uwr->extensions.stopped = param + strlen(JK_UWMAP_EXTENSION_STOPPED);
+                }
+                else if (!strncmp(param, JK_UWMAP_EXTENSION_FAIL_ON_STATUS, strlen(JK_UWMAP_EXTENSION_FAIL_ON_STATUS))) {
+                    if (uwr->extensions.fail_on_status_str)
+                        jk_log(l, JK_LOG_WARNING,
+                               "extension '%s' in uri worker map only allowed once",
+                               JK_UWMAP_EXTENSION_FAIL_ON_STATUS);
+                    else
+                        uwr->extensions.fail_on_status_str = param + strlen(JK_UWMAP_EXTENSION_FAIL_ON_STATUS);
+                }
+                else {
+                    jk_log(l, JK_LOG_WARNING,
+                           "unknown extension '%s' in uri worker map",
+                           param);
+                }
+            }
+        }
+
+        uwr->source_type = source_type;
+        uwr->worker_name = w;
         uwr->uri = uri;
         uwr->context = uri;
-        uwr->worker_name = jk_pool_strdup(&uw_map->p, worker);
         uwr->context_len = strlen(uwr->context);
-        uwr->source_type = source_type;
         if (strchr(uri, '*') ||
             strchr(uri, '?')) {
             /* Something like
@@ -376,17 +716,19 @@ int uri_worker_map_add(jk_uri_worker_map_t *uw_map,
              * /context/ *.suffix
              */
             match_type |= MATCH_TYPE_WILDCHAR_PATH;
-            jk_log(l, JK_LOG_DEBUG,
-                   "wildchar rule '%s=%s' source '%s' was added",
-                   uwr->context, uwr->worker_name, uri_worker_map_get_source(uwr, l));
+            if (JK_IS_DEBUG_LEVEL(l))
+                jk_log(l, JK_LOG_DEBUG,
+                       "wildchar rule '%s=%s' source '%s' was added",
+                       uwr->context, uwr->worker_name, uri_worker_map_get_source(uwr, l));
 
         }
         else {
             /* Something like:  JkMount /login/j_security_check ajp13 */
             match_type |= MATCH_TYPE_EXACT;
-            jk_log(l, JK_LOG_DEBUG,
-                   "exact rule '%s=%s' source '%s' was added",
-                   uwr->context, uwr->worker_name, uri_worker_map_get_source(uwr, l));
+            if (JK_IS_DEBUG_LEVEL(l))
+                jk_log(l, JK_LOG_DEBUG,
+                       "exact rule '%s=%s' source '%s' was added",
+                       uwr->context, uwr->worker_name, uri_worker_map_get_source(uwr, l));
         }
     }
     else {
@@ -403,11 +745,11 @@ int uri_worker_map_add(jk_uri_worker_map_t *uw_map,
         return JK_FALSE;
     }
     uwr->match_type = match_type;
-    uw_map->maps[uw_map->size] = uwr;
-    uw_map->size++;
+    IND_NEXT(uw_map->maps)[IND_NEXT(uw_map->size)] = uwr;
+    IND_NEXT(uw_map->size)++;
     if (match_type & MATCH_TYPE_NO_MATCH) {
         /* If we split the mappings this one will be calculated */
-        uw_map->nosize++;
+        IND_NEXT(uw_map->nosize)++;
     }
     worker_qsort(uw_map);
     JK_TRACE_EXIT(l);
@@ -424,9 +766,10 @@ int uri_worker_map_open(jk_uri_worker_map_t *uw_map,
     if (uw_map) {
         int sz = jk_map_size(init_data);
 
-        jk_log(l, JK_LOG_DEBUG,
-               "rule map size is %d",
-               sz);
+        if (JK_IS_DEBUG_LEVEL(l))
+            jk_log(l, JK_LOG_DEBUG,
+                   "rule map size is %d",
+                   sz);
 
         if (sz > 0) {
             int i;
@@ -475,24 +818,28 @@ int uri_worker_map_open(jk_uri_worker_map_t *uw_map,
 
         if (rc == JK_FALSE) {
             jk_log(l, JK_LOG_ERROR,
-                   "there was an error, freing buf");
+                   "there was an error, freeing buf");
+            jk_close_pool(&uw_map->p_dyn[0]);
+            jk_close_pool(&uw_map->p_dyn[1]);
             jk_close_pool(&uw_map->p);
         }
+        else if (JK_IS_DEBUG_LEVEL(l))
+            uri_worker_map_dump(uw_map, "after map open", l);
     }
 
     JK_TRACE_EXIT(l);
     return rc;
 }
 
-static const char *find_match(jk_uri_worker_map_t *uw_map,
-                        const char *url, jk_logger_t *l)
+static int find_match(jk_uri_worker_map_t *uw_map,
+                      const char *url, jk_logger_t *l)
 {
     unsigned int i;
 
     JK_TRACE_ENTER(l);
 
-    for (i = 0; i < uw_map->size; i++) {
-        uri_worker_record_t *uwr = uw_map->maps[i];
+    for (i = 0; i < IND_THIS(uw_map->size); i++) {
+        uri_worker_record_t *uwr = IND_THIS(uw_map->maps)[i];
 
         /* Check for match types */
         if ((uwr->match_type & MATCH_TYPE_DISABLED) ||
@@ -505,7 +852,7 @@ static const char *find_match(jk_uri_worker_map_t *uw_map,
 
         if (uwr->match_type & MATCH_TYPE_WILDCHAR_PATH) {
             /* Map is already sorted by context_len */
-            if (wildchar_match(url, uwr->context,
+            if (jk_wildchar_match(url, uwr->context,
 #ifdef WIN32
                                0
 #else
@@ -517,7 +864,7 @@ static const char *find_match(jk_uri_worker_map_t *uw_map,
                                "Found a wildchar match '%s=%s'",
                                uwr->context, uwr->worker_name);
                     JK_TRACE_EXIT(l);
-                    return uwr->worker_name;
+                    return i;
              }
         }
         else if (JK_STRNCMP(uwr->context, url, uwr->context_len) == 0) {
@@ -527,45 +874,48 @@ static const char *find_match(jk_uri_worker_map_t *uw_map,
                            "Found an exact match '%s=%s'",
                            uwr->context, uwr->worker_name);
                 JK_TRACE_EXIT(l);
-                return uwr->worker_name;
+                return i;
             }
         }
     }
 
     JK_TRACE_EXIT(l);
-    return NULL;
+    return -1;
 }
 
 static int is_nomatch(jk_uri_worker_map_t *uw_map,
-                      const char *uri, const char* worker,
+                      const char *uri, int match,
                       jk_logger_t *l)
 {
     unsigned int i;
+    const char *worker = IND_THIS(uw_map->maps)[match]->worker_name;
 
     JK_TRACE_ENTER(l);
 
-    for (i = 0; i < uw_map->size; i++) {
-        uri_worker_record_t *uwr = uw_map->maps[i];
+    for (i = 0; i < IND_THIS(uw_map->size); i++) {
+        uri_worker_record_t *uwr = IND_THIS(uw_map->maps)[i];
 
         /* Check only nomatch mappings */
         if (!(uwr->match_type & MATCH_TYPE_NO_MATCH) ||
             (uwr->match_type & MATCH_TYPE_DISABLED))
             continue;
         /* Check only matching workers */
-        if (*uwr->worker_name != '*' && strcmp(uwr->worker_name, worker))
+        if (strcmp(uwr->worker_name, worker) &&
+            strcmp(uwr->worker_name, "*"))
             continue;
         if (uwr->match_type & MATCH_TYPE_WILDCHAR_PATH) {
             /* Map is already sorted by context_len */
-            if (wildchar_match(uri, uwr->context,
+            if (jk_wildchar_match(uri, uwr->context,
 #ifdef WIN32
                                0
 #else
                                0
 #endif
                                ) == 0) {
-                    jk_log(l, JK_LOG_DEBUG,
-                           "Found a wildchar no match '%s=%s' source '%s'",
-                           uwr->context, uwr->worker_name, uri_worker_map_get_source(uwr, l));
+                    if (JK_IS_DEBUG_LEVEL(l))
+                        jk_log(l, JK_LOG_DEBUG,
+                               "Found a wildchar no match '%s=%s' source '%s'",
+                               uwr->context, uwr->worker_name, uri_worker_map_get_source(uwr, l));
                     JK_TRACE_EXIT(l);
                     return JK_TRUE;
              }
@@ -586,24 +936,28 @@ static int is_nomatch(jk_uri_worker_map_t *uw_map,
     return JK_FALSE;
 }
 
-
-const char *map_uri_to_worker(jk_uri_worker_map_t *uw_map,
-                              const char *uri, const char *vhost,
-                              jk_logger_t *l)
+const char *map_uri_to_worker_ext(jk_uri_worker_map_t *uw_map,
+                                  const char *uri, const char *vhost,
+                                  rule_extension_t **extensions,
+                                  int *index,
+                                  jk_logger_t *l)
 {
     unsigned int i;
     unsigned int vhost_len;
     int reject_unsafe;
-    const char *rv = NULL;
+    int rv = -1;
     char  url[JK_MAX_URI_LEN+1];
 
     JK_TRACE_ENTER(l);
 
-    if (!uw_map || !uri) {
+    if (!uw_map || !uri || !extensions) {
         JK_LOG_NULL_PARAMS(l);
         JK_TRACE_EXIT(l);
         return NULL;
     }
+    *extensions = NULL;
+    if (index)
+        *index = -1;
     if (*uri != '/') {
         jk_log(l, JK_LOG_WARNING,
                 "Uri %s is invalid. Uri must start with /", uri);
@@ -612,7 +966,7 @@ const char *map_uri_to_worker(jk_uri_worker_map_t *uw_map,
     }
     if (uw_map->fname) {
         uri_worker_map_update(uw_map, 0, l);
-        if (!uw_map->size) {
+        if (!IND_THIS(uw_map->size)) {
             jk_log(l, JK_LOG_INFO,
                    "No worker maps defined for %s.",
                    uw_map->fname);
@@ -625,19 +979,19 @@ const char *map_uri_to_worker(jk_uri_worker_map_t *uw_map,
      */
     reject_unsafe = uw_map->reject_unsafe;
     vhost_len = 0;
-/*
- * In case we got a vhost, we prepend a slash
- * and the vhost to the url in order to enable
- * vhost mapping rules especially for IIS.
- */
+    /*
+     * In case we got a vhost, we prepend a slash
+     * and the vhost to the url in order to enable
+     * vhost mapping rules especially for IIS.
+     */
     if (vhost) {
         int off = 0;
-/* Add leading '/' if necessary */
+        /* Add leading '/' if necessary */
         if (vhost[0] != '/') {
             url[0] = '/';
             off = 1;
         }
-/* Size including leading slash. */
+        /* Size including leading slash. */
         vhost_len = strlen(vhost);
         if (vhost_len + off >= JK_MAX_URI_LEN) {
             vhost_len = 0;
@@ -680,32 +1034,58 @@ const char *map_uri_to_worker(jk_uri_worker_map_t *uw_map,
     }
     if (JK_IS_DEBUG_LEVEL(l))
         jk_log(l, JK_LOG_DEBUG, "Attempting to map URI '%s' from %d maps",
-               url, uw_map->size);
+               url, IND_THIS(uw_map->size));
     rv = find_match(uw_map, url, l);
-/* If this doesn't find a match, try without the vhost. */
-    if (! rv && vhost_len) {
+    /* If this doesn't find a match, try without the vhost. */
+    if (rv < 0 && vhost_len) {
         rv = find_match(uw_map, &url[vhost_len], l);
     }
 
-/* In case we found a match, check for the unmounts. */
-    if (rv && uw_map->nosize) {
-/* Again first including vhost. */
+    /* In case we found a match, check for the unmounts. */
+    if (rv >= 0 && IND_THIS(uw_map->nosize)) {
+        /* Again first including vhost. */
         int rc = is_nomatch(uw_map, url, rv, l);
-/* If no unmount was find, try without vhost. */
+        /* If no unmount was find, try without vhost. */
         if (!rc && vhost_len)
             rc = is_nomatch(uw_map, &url[vhost_len], rv, l);
         if (rc) {
             if (JK_IS_DEBUG_LEVEL(l)) {
                 jk_log(l, JK_LOG_DEBUG,
                        "Denying match for worker %s by nomatch rule",
-                       rv);
+                       IND_THIS(uw_map->maps)[rv]->worker_name);
             }
-            rv = NULL;
+            rv = -1;
         }
     }
 
+    if (rv >= 0) {
+        *extensions = &(IND_THIS(uw_map->maps)[rv]->extensions);
+        if (index)
+            *index = rv;
+        JK_TRACE_EXIT(l);
+        return IND_THIS(uw_map->maps)[rv]->worker_name;
+    }
     JK_TRACE_EXIT(l);
-    return rv;
+    return NULL;
+}
+
+rule_extension_t *get_uri_to_worker_ext(jk_uri_worker_map_t *uw_map,
+                                        int index)
+{
+    if (index >= 0) {
+        return &(IND_THIS(uw_map->maps)[index]->extensions);
+    }
+    else {
+        return NULL;
+    }
+}
+
+const char *map_uri_to_worker(jk_uri_worker_map_t *uw_map,
+                              const char *uri, const char *vhost,
+                              jk_logger_t *l)
+{
+    rule_extension_t *ext;
+    return map_uri_to_worker_ext(uw_map, uri, vhost, &ext, NULL, l);
 }
 
 int uri_worker_map_load(jk_uri_worker_map_t *uw_map,
@@ -715,14 +1095,14 @@ int uri_worker_map_load(jk_uri_worker_map_t *uw_map,
     jk_map_t *map;
 
     jk_map_alloc(&map);
-    if (jk_map_read_properties(map, uw_map->fname, &uw_map->modified,
+    if (jk_map_read_properties(map, NULL, uw_map->fname, &uw_map->modified,
                                JK_MAP_HANDLE_NORMAL, l)) {
         int i;
         if (JK_IS_DEBUG_LEVEL(l))
             jk_log(l, JK_LOG_DEBUG,
                    "Loading urimaps from %s with reload check interval %d seconds",
                    uw_map->fname, uw_map->reload);
-        uri_worker_map_clear(uw_map, SOURCE_TYPE_URIMAP, l);
+        uri_worker_map_clear(uw_map, l);
         for (i = 0; i < jk_map_size(map); i++) {
             const char *u = jk_map_name_at(map, i);
             const char *w = jk_map_value_at(map, i);
@@ -759,6 +1139,8 @@ int uri_worker_map_load(jk_uri_worker_map_t *uw_map,
             }
         }
         uw_map->checked = time(NULL);
+        if (JK_IS_DEBUG_LEVEL(l))
+            uri_worker_map_dump(uw_map, "after file load", l);
         rc = JK_TRUE;
     }
     jk_map_free(&map);
@@ -771,8 +1153,8 @@ int uri_worker_map_update(jk_uri_worker_map_t *uw_map,
     int rc = JK_TRUE;
     time_t now = time(NULL);
 
-    if ((uw_map->reload > 0 && difftime(now, uw_map->checked) > uw_map->reload) ||
-        force) {
+    if (force || (uw_map->reload > 0 && difftime(now, uw_map->checked) >
+                  uw_map->reload)) {
         struct stat statbuf;
         uw_map->checked = now;
         if ((rc = jk_stat(uw_map->fname, &statbuf)) == -1) {
@@ -784,7 +1166,7 @@ int uri_worker_map_update(jk_uri_worker_map_t *uw_map,
         if (statbuf.st_mtime == uw_map->modified) {
             if (JK_IS_DEBUG_LEVEL(l))
                 jk_log(l, JK_LOG_DEBUG,
-                       "File %s  is not modified",
+                       "File %s is not modified",
                        uw_map->fname);
             return JK_TRUE;
         }
@@ -799,6 +1181,8 @@ int uri_worker_map_update(jk_uri_worker_map_t *uw_map,
             return JK_TRUE;
         }
         rc = uri_worker_map_load(uw_map, l);
+        uri_worker_map_ext(uw_map, l);
+        uri_worker_map_switch(uw_map, l);
         JK_LEAVE_CS(&(uw_map->cs), rc);
         jk_log(l, JK_LOG_INFO,
                "Reloaded urimaps from %s", uw_map->fname);
