@@ -20,7 +20,7 @@
  * Author:      Gal Shachor <shachor@il.ibm.com>                           *
  * Author:      Henri Gomez <hgomez@apache.org>                            *
  * Author:      Rainer Jung <rjung@apache.org>                             *
- * Version:     $Revision: 752016 $                                          *
+ * Version:     $Revision: 915199 $                                          *
  ***************************************************************************/
 
 
@@ -320,7 +320,8 @@ static const char *supported_properties[] = {
     RETRIES_OF_WORKER,
     STATUS_FAIL_OF_WORKER,
     LIST_PROPERTY_NAME,
-    MAINTAIN_PROPERTY_NAME
+    MAINTAIN_PROPERTY_NAME,
+    NULL
 };
 
 static const char *jk_level_verbs[] = {
@@ -541,9 +542,21 @@ static int JK_METHOD log_to_file(jk_logger_t *l, int level, int used, char *what
         if (p->logfile) {
             what[used++] = '\n';
             what[used] = '\0';
+#if defined(JK_LOG_LOCKING)
+#if defined(WIN32) && defined(_MSC_VER)
+            LockFile((HANDLE)_get_osfhandle(_fileno(p->logfile)),
+                     0, 0, 1, 0);
+#endif
+#endif
             fputs(what, p->logfile);
             /* [V] Flush the dam' thing! */
             fflush(p->logfile);
+#if defined(JK_LOG_LOCKING)
+#if defined(WIN32) && defined(_MSC_VER)
+            UnlockFile((HANDLE)_get_osfhandle(_fileno(p->logfile)),
+                       0, 0, 1, 0);
+#endif
+#endif
         }
         return JK_TRUE;
     }
@@ -591,6 +604,8 @@ int jk_open_file_logger(jk_logger_t **l, const char *file, int level)
             rc->logger_private = p;
 #if defined(AS400) && !defined(AS400_UTF8)
             p->logfile = fopen(file, "a+, o_ccsid=0");
+#elif defined(WIN32) && defined(_MSC_VER)
+            p->logfile = fopen(file, "a+c");
 #else
             p->logfile = fopen(file, "a+");
 #endif
@@ -624,6 +639,8 @@ int jk_attach_file_logger(jk_logger_t **l, int fd, int level)
             rc->logger_private = p;
 #if defined(AS400) && !defined(AS400_UTF8)
             p->logfile = fdopen(fd, "a+, o_ccsid=0");
+#elif defined(WIN32) && defined(_MSC_VER)
+            p->logfile = fdopen(fd, "a+c");
 #else
             p->logfile = fdopen(fd, "a+");
 #endif
@@ -709,7 +726,7 @@ int jk_log(jk_logger_t *l,
             /* Performance is no issue, because with production log levels */
             /* we only call it often, if we have a lot of errors */
             rc = snprintf(buf + used, usable_size - used,
-                          "[%" JK_PID_T_FMT ":%" JK_UINT32_T_FMT "] ", getpid(), jk_gettid());
+                          "[%" JK_PID_T_FMT ":%" JK_PTHREAD_T_FMT "] ", getpid(), jk_gettid());
             used += rc;
             if (rc < 0 ) {
                 return 0;
@@ -2108,7 +2125,7 @@ int jk_wildchar_match(const char *str, const char *exp, int icase)
 }
 
 #ifdef _MT_CODE_PTHREAD
-jk_uint32_t jk_gettid()
+jk_pthread_t jk_gettid()
 {
     union {
         pthread_t tid;
@@ -2124,14 +2141,7 @@ jk_uint32_t jk_gettid()
     pthread_getunique_np(&(u.tid), &tid);
     return ((jk_uint32_t)(tid.intId.lo & 0xFFFFFFFF));
 #else
-    switch(sizeof(pthread_t)) {
-    case sizeof(jk_uint32_t):
-        return *(jk_uint32_t *)&u.tid;
-    case sizeof(jk_uint64_t):
-        return (*(jk_uint64_t *)&u.tid) & 0xFFFFFFFF;
-    default:
-        return 0;
-    }
+    return ((jk_pthread_t)u.tid);
 #endif /* AS400 */
 }
 #endif
