@@ -20,7 +20,7 @@
  * Author:      Gal Shachor <shachor@il.ibm.com>                           *
  * Author:      Henri Gomez <hgomez@apache.org>                            *
  * Author:      Rainer Jung <rjung@apache.org>                             *
- * Version:     $Revision: 596813 $                                          *
+ * Version:     $Revision: 751914 $                                          *
  ***************************************************************************/
 #ifndef _JK_UTIL_H
 #define _JK_UTIL_H
@@ -45,6 +45,8 @@ int jk_parse_log_level(const char *level);
 
 int jk_open_file_logger(jk_logger_t **l, const char *file, int level);
 
+int jk_attach_file_logger(jk_logger_t **l, int fd, int level);
+
 int jk_close_file_logger(jk_logger_t **l);
 
 int jk_log(jk_logger_t *l,
@@ -68,12 +70,18 @@ int jk_get_worker_cache_size(jk_map_t *m, const char *wname, int def);
 
 int jk_get_worker_cache_size_min(jk_map_t *m, const char *wname, int def);
 
+int jk_get_worker_cache_acquire_timeout(jk_map_t *m, const char *wname, int def);
+
 int jk_get_worker_socket_timeout(jk_map_t *m, const char *wname, int def);
 
+int jk_get_worker_socket_connect_timeout(jk_map_t *m, const char *wname, int def);
+
 int jk_get_worker_socket_buffer(jk_map_t *m, const char *wname, int def);
 
 int jk_get_worker_socket_keepalive(jk_map_t *m, const char *wname, int def);
 
+int jk_get_worker_conn_ping_interval(jk_map_t *m, const char *wname, int def);
+
 int jk_get_worker_cache_timeout(jk_map_t *m, const char *wname, int def);
 
 int jk_get_worker_recovery_opts(jk_map_t *m, const char *wname, int def);
@@ -84,12 +92,20 @@ int jk_get_worker_reply_timeout(jk_map_t *m, const char *wname, int def);
 
 int jk_get_worker_prepost_timeout(jk_map_t *m, const char *wname, int def);
 
+int jk_get_worker_ping_timeout(jk_map_t *m, const char *wname, int def);
+
+int jk_get_worker_ping_mode(jk_map_t *m, const char *wname, int def);
+
 int jk_get_worker_recycle_timeout(jk_map_t *m, const char *wname, int def);
 
 int jk_get_worker_recover_timeout(jk_map_t *m, const char *wname, int def);
 
+int jk_get_worker_error_escalation_time(jk_map_t *m, const char *wname, int def);
+
 int jk_get_worker_max_reply_timeouts(jk_map_t *m, const char *wname, int def);
 
+int jk_get_worker_retry_interval(jk_map_t *m, const char *wname, int def);
+
 const char *jk_get_worker_route(jk_map_t *m, const char *wname, const char *def);
 
 const char *jk_get_worker_domain(jk_map_t *m, const char *wname, const char *def);
@@ -98,6 +114,10 @@ const char *jk_get_worker_redirect(jk_map_t *m, const char *wname, const char *d
 
 const char *jk_get_worker_secret_key(jk_map_t *m, const char *wname);
 
+const char *jk_get_lb_session_cookie(jk_map_t *m, const char *wname, const char *def);
+
+const char *jk_get_lb_session_path(jk_map_t *m, const char *wname, const char *def);
+
 int jk_get_worker_retries(jk_map_t *m, const char *wname, int def);
 
 int jk_get_is_worker_disabled(jk_map_t *m, const char *wname);
@@ -206,9 +226,13 @@ const char *jk_get_worker_prop_prefix(jk_map_t *m, const char *wname, const char
 int jk_get_worker_fail_on_status(jk_map_t *m, const char *wname,
                                  int *list, unsigned int list_size);
 
-
 int jk_get_worker_user_case_insensitive(jk_map_t *m, const char *wname);
 
+int is_http_status_fail(unsigned int http_status_fail_num,
+                        int *http_status_fail, int status);
+
+int jk_wildchar_match(const char *str, const char *exp, int icase);
+
 #define TC32_BRIDGE_TYPE    32
 #define TC33_BRIDGE_TYPE    33
 #define TC40_BRIDGE_TYPE    40
