@@ -18,7 +18,7 @@
 /***************************************************************************
  * Description: NSAPI plugin for Netscape servers                          *
  * Author:      Gal Shachor <shachor@il.ibm.com>                           *
- * Version:     $Revision: 604859 $                                           *
+ * Version:     $Revision: 756058 $                                           *
  ***************************************************************************/
 
 
@@ -59,6 +59,7 @@ static jk_logger_t *logger = NULL;
 static jk_worker_env_t worker_env;
 static jk_map_t *init_map = NULL;
 static jk_uri_worker_map_t *uw_map = NULL;
+static size_t jk_shm_size = 0;
 
 #ifdef NETWARE
 int (*PR_IsSocketSecure) (SYS_NETFD * csd);     /* pointer to PR_IsSocketSecure function */
@@ -98,8 +99,12 @@ static void init_workers_on_other_threads(void *init_d)
         uw_map->reload = JK_URIMAP_DEF_RELOAD;
         uw_map->reject_unsafe = jk_map_get_bool(init_map, "worker." REJECT_UNSAFE_TAG, JK_FALSE);
         worker_env.uri_to_worker = uw_map;
+        worker_env.pool = NULL;
+
         if (wc_open(init_map, &worker_env, logger)) {
             init_on_other_thread_is_ok = JK_TRUE;
+            uri_worker_map_ext(uw_map, logger);
+            uri_worker_map_switch(uw_map, logger);
         }
         else {
             jk_log(logger, JK_LOG_EMERG,
@@ -159,7 +164,6 @@ static int JK_METHOD ws_read(jk_ws_service_t *s,
 
         *a = 0;
         if (l) {
-            char *buf = b;
             unsigned i;
             netbuf *inbuf = p->sn->inbuf;
 
@@ -172,6 +176,7 @@ static int JK_METHOD ws_read(jk_ws_service_t *s,
             }
 
 #else
+            char *buf = b;
             int ch;
             for (i = 0; i < l; i++) {
                 ch = netbuf_getc(inbuf);
@@ -268,10 +273,10 @@ NSAPI_PUBLIC int jk_init(pblock * pb, Session * sn, Request * rq)
         logger = NULL;
     }
     
-    jk_shm_open(shm_file, JK_SHM_DEF_SIZE, logger);
     if (jk_map_alloc(&init_map)) {
-        if (jk_map_read_properties(init_map, worker_prp_file, NULL,
+        if (jk_map_read_properties(init_map, NULL, worker_prp_file, NULL,
                                    JK_MAP_HANDLE_DUPLICATES, logger)) {
+            int rv;
             int sleep_cnt;
             SYS_THREAD s;
 
@@ -282,6 +287,13 @@ NSAPI_PUBLIC int jk_init(pblock * pb, Session * sn, Request * rq)
             if (reject_unsafe) {
                 jk_map_add(init_map, "worker." REJECT_UNSAFE_TAG, reject_unsafe); 
             }
+
+            jk_shm_size = jk_shm_calculate_size(init_map, logger);
+            if ((rv = jk_shm_open(shm_file, jk_shm_size, logger)) != 0)
+                jk_log(logger, JK_LOG_ERROR,
+                       "Initializing shm:%s errno=%d. Load balancing workers will not function properly.",
+                       jk_shm_name(), rv);
+
             s = systhread_start(SYSTHREAD_DEFAULT_PRIORITY,
                                 0, init_workers_on_other_threads, init_map);
             for (sleep_cnt = 0; sleep_cnt < 60; sleep_cnt++) {
@@ -295,8 +307,7 @@ NSAPI_PUBLIC int jk_init(pblock * pb, Session * sn, Request * rq)
             if (init_on_other_thread_is_done && init_on_other_thread_is_ok) {
                 magnus_atrestart(jk_term, NULL);
                 rc = REQ_PROCEED;
-                jk_log(logger, JK_LOG_INFO, "nsapi_redirector/%s initialized",
-                       JK_VERSTRING);
+                jk_log(logger, JK_LOG_INFO, "%s initialized", JK_EXPOSED_VERSION);
             }
 
 /*            if(wc_open(init_map, NULL, logger)) {
@@ -372,7 +383,6 @@ NSAPI_PUBLIC int jk_service(pblock * pb, Session * sn, Request * rq)
 
         s.ws_private = &private_data;
         s.pool = &private_data.p;
-        s.retries = worker->retries;
 
         wc_maintain(logger);
         if (init_ws_service(&private_data, &s)) {
@@ -416,22 +426,13 @@ static int init_ws_service(nsapi_private_data_t * private_data,
     int size;
     int rc;
 
-    s->route = NULL;
     s->start_response = start_response;
     s->read = ws_read;
     s->write = ws_write;
-    s->flush = NULL;
-    s->flush_packets = JK_FALSE;
-    s->flush_header = JK_FALSE;
-    s->disable_reuse = JK_FALSE;
-
-    /* Clear RECO status */
-    s->reco_status = RECO_NONE;
 
     s->auth_type = pblock_findval("auth-type", private_data->rq->vars);
     s->remote_user = pblock_findval("auth-user", private_data->rq->vars);
 
-    s->content_length = 0;
     tmp = NULL;
     rc = request_header("content-length",
                         &tmp, private_data->sn, private_data->rq);
@@ -467,10 +468,6 @@ static int init_ws_service(nsapi_private_data_t * private_data,
         s->server_port = server_portnum;
     s->server_software = system_version();
 
-
-    s->headers_names = NULL;
-    s->headers_values = NULL;
-    s->num_headers = 0;
     s->uw_map = uw_map;
 
 #ifdef NETWARE
@@ -484,7 +481,6 @@ static int init_ws_service(nsapi_private_data_t * private_data,
 #endif
         s->is_ssl = security_active;
 
-    s->ssl_key_size = -1;       /* required by Servlet 2.3 Api, added in jtc */
     if (s->is_ssl) {
         char *ssl_cert = pblock_findval("auth-cert", private_data->rq->vars);
         if (ssl_cert != NULL) {
@@ -498,12 +494,7 @@ static int init_ws_service(nsapi_private_data_t * private_data,
         }
         s->ssl_cipher = pblock_findval("cipher", private_data->sn->client);
         s->ssl_session = pblock_findval("ssl-id", private_data->sn->client);
-    }
-    else {
-        s->ssl_cert = NULL;
-        s->ssl_cert_len = 0;
-        s->ssl_cipher = NULL;
-        s->ssl_session = NULL;
+        /* XXX: We need to investigate how to set s->ssl_key_size */
     }
 
     rc = setup_http_headers(private_data, s);
